@@ -1,12 +1,36 @@
+from __future__ import annotations
+
 import functools
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Generator,
+    Generic,
+    Iterable,
+    Optional,
+    TypeVar,
+    cast,
+)
+
+if TYPE_CHECKING:
+    from typing_extensions import TypeAlias
+
+    from pykka._types import OptExcInfo
 
 __all__ = ["Future", "get_all"]
 
 
-class Future:
-    """
-    A :class:`Future` is a handle to a value which is available or will be
-    available in the future.
+T = TypeVar("T")
+J = TypeVar("J")  # For when T is Iterable[J]
+M = TypeVar("M")  # Result of Future.map()
+R = TypeVar("R")  # Result of Future.reduce()
+
+GetHookFunc: TypeAlias = Callable[[Optional[float]], T]
+
+
+class Future(Generic[T]):
+    """A handle to a value which is available now or in the future.
 
     Typically returned by calls to actor methods or accesses to actor fields.
 
@@ -14,14 +38,23 @@ class Future:
     ``await`` the future.
     """
 
-    def __init__(self):
+    _get_hook: Optional[GetHookFunc[T]]
+    _get_hook_result: Optional[T]
+
+    def __init__(self) -> None:
         super().__init__()
         self._get_hook = None
         self._get_hook_result = None
 
-    def get(self, timeout=None):
-        """
-        Get the value encapsulated by the future.
+    def __repr__(self) -> str:
+        return "<pykka.Future>"
+
+    def get(
+        self,
+        *,
+        timeout: Optional[float] = None,
+    ) -> T:
+        """Get the value encapsulated by the future.
 
         If the encapsulated value is an exception, it is raised instead of
         returned.
@@ -47,9 +80,11 @@ class Future:
             return self._get_hook_result
         raise NotImplementedError
 
-    def set(self, value=None):
-        """
-        Set the encapsulated value.
+    def set(
+        self,
+        value: Optional[T] = None,
+    ) -> None:
+        """Set the encapsulated value.
 
         :param value: the encapsulated value or nothing
         :type value: any object or :class:`None`
@@ -57,9 +92,11 @@ class Future:
         """
         raise NotImplementedError
 
-    def set_exception(self, exc_info=None):
-        """
-        Set an exception as the encapsulated value.
+    def set_exception(
+        self,
+        exc_info: Optional[OptExcInfo] = None,
+    ) -> None:
+        """Set an exception as the encapsulated value.
 
         You can pass an ``exc_info`` three-tuple, as returned by
         :func:`sys.exc_info`. If you don't pass ``exc_info``,
@@ -74,9 +111,11 @@ class Future:
         """
         raise NotImplementedError
 
-    def set_get_hook(self, func):
-        """
-        Set a function to be executed when :meth:`get` is called.
+    def set_get_hook(
+        self,
+        func: GetHookFunc[T],
+    ) -> None:
+        """Set a function to be executed when :meth:`get` is called.
 
         The function will be called when :meth:`get` is called, with the
         ``timeout`` value as the only argument. The function's return value
@@ -89,9 +128,11 @@ class Future:
         """
         self._get_hook = func
 
-    def filter(self, func):
-        """
-        Return a new future with only the items passing the predicate function.
+    def filter(
+        self: Future[Iterable[J]],
+        func: Callable[[J], bool],
+    ) -> Future[Iterable[J]]:
+        """Return a new future with only the items passing the predicate function.
 
         If the future's value is an iterable, :meth:`filter` will return a new
         future whose value is another iterable with only the items from the
@@ -115,12 +156,16 @@ class Future:
         .. versionadded:: 1.2
         """
         future = self.__class__()
-        future.set_get_hook(lambda timeout: list(filter(func, self.get(timeout))))
+        future.set_get_hook(
+            lambda timeout: list(filter(func, self.get(timeout=timeout)))
+        )
         return future
 
-    def join(self, *futures):
-        """
-        Return a new future with a list of the result of multiple futures.
+    def join(
+        self: Future[Any],
+        *futures: Future[Any],
+    ) -> Future[Iterable[Any]]:
+        """Return a new future with a list of the result of multiple futures.
 
         One or more futures can be passed as arguments to :meth:`join`. The new
         future returns a list with the results from all the joined futures.
@@ -140,16 +185,17 @@ class Future:
 
         .. versionadded:: 1.2
         """
-        future = self.__class__()
+        future = cast(Future[Iterable[Any]], self.__class__())
         future.set_get_hook(
-            lambda timeout: [f.get(timeout) for f in [self] + list(futures)]
+            lambda timeout: [f.get(timeout=timeout) for f in [self, *futures]]
         )
         return future
 
-    def map(self, func):
-        """
-        Return a new future with the result of the future passed through a
-        function.
+    def map(
+        self,
+        func: Callable[[T], M],
+    ) -> Future[M]:
+        """Pass the result of the future through a function.
 
         Example::
 
@@ -176,16 +222,16 @@ class Future:
             behavior has been simplified. Now, the entire result value is
             passed to the function.
         """
-        future = self.__class__()
-        future.set_get_hook(lambda timeout: func(self.get(timeout)))
+        future = cast(Future[M], self.__class__())
+        future.set_get_hook(lambda timeout: func(self.get(timeout=timeout)))
         return future
 
-    def reduce(self, func, *args):
-        """
-        reduce(func[, initial])
-
-        Return a new future with the result of reducing the future's iterable
-        into a single value.
+    def reduce(
+        self: Future[Iterable[J]],
+        func: Callable[[R, J], R],
+        *args: R,
+    ) -> Future[R]:
+        """Reduce a future's iterable result to a single value.
 
         The function of two arguments is applied cumulatively to the items of
         the iterable, from left to right. The result of the first function call
@@ -230,13 +276,13 @@ class Future:
 
         .. versionadded:: 1.2
         """
-        future = self.__class__()
+        future = cast(Future[R], self.__class__())
         future.set_get_hook(
-            lambda timeout: functools.reduce(func, self.get(timeout), *args)
+            lambda timeout: functools.reduce(func, self.get(timeout=timeout), *args)
         )
         return future
 
-    def __await__(self):
+    def __await__(self) -> Generator[None, None, T]:
         yield
         value = self.get()
         return value
@@ -244,9 +290,12 @@ class Future:
     __iter__ = __await__
 
 
-def get_all(futures, timeout=None):
-    """
-    Collect all values encapsulated in the list of futures.
+def get_all(
+    futures: Iterable[Future[T]],
+    *,
+    timeout: Optional[float] = None,
+) -> Iterable[T]:
+    """Collect all values encapsulated in the list of futures.
 
     If ``timeout`` is not :class:`None`, the method will wait for a reply for
     ``timeout`` seconds, and then raise :exc:`pykka.Timeout`.
